hwe.cnv <- function(object,group=NULL,fdr=FALSE) {
  ## HWE test of 'object', optionally by group 
  ##..object = an object of class 'cnv'; output of setreg.cnv
  ##..group = the position of group variable in 'pheno.data' data.frame in 
  ##..........default=NULL -> summarize all CNV dels/dups regions simultaneously.
  ## fdr = whether to adjust the pvalues or not.
 
   both <- attr(object,"both")
   if(!both)
      stop("HWE can only be carried out with cnv.abnormality==both when using setreg.cnv")

  require(gap)  

  if(is.null(group)) {
    grp = c(paste("group=","ALL",sep=""))
    ng = 1
  }  
  
  if(!is.null(group)) {
    ng = length(table(object$pheno[,group]))
    grp.var = names(table(object$pheno[,group]))
    #class(grp.var) = class(object$pheno[,group])
    grp = c(paste("group=",grp.var,sep=""))
    
  }
 #... start of HWE test for diallelic 
 cat('\n','HWE test group-by-group:','\n') 
  hwe.reg = hwe.regtot = list()
  FREQ =  FDR = MAF = list()
  for (i in 1:ng){
    pick = rep(TRUE,length(object[["reg"]][["regID"]]))
    nsubj = sum(pick)

    if(!is.null(group)) {
      pick = object$pheno[,group]==grp.var[i]
      nsubj = sum(pick)
    }

    del = unlist(object[["reg"]][["regID"]][pick])   # positions in the LOC.del list
    freq = table(as.numeric(del))
    FREQ[[i]] = data.frame(freq)
    pval = NULL
    maf = NULL
    udel = sort(unique(as.numeric(del)))
    for(j in udel) {
      cn = unlist(object[["reg"]][["call"]][pick])
      cn = cn[as.numeric(del)==j]
      freq = table(cn)
      cn.cat = as.numeric(names(freq))

      if(min(cn.cat)<2 & max(cn.cat)>2) {  #multi-allelic region, we don't test HWE
	pval = c(pval,NA)
	maf = c(maf,NA)
      }
      if(min(cn.cat)<=2 & max(cn.cat)<=2) { #di-allelic region, del only
	x = rep(0,3); x[cn.cat+1] = freq	
	x[3] = nsubj-sum(x) #subj with normal CN
	x.hwe = hwe(x,data.type='count')
	pval = c(pval,x.hwe$p.lrt)
	maf = c(maf,min(x.hwe$allele.freq))
      }
      if(min(cn.cat)>=2 & max(cn.cat)>=2) { #di-allelic region, dups only
	x = rep(0,3); x[cn.cat-1] = freq	
	x[1] = nsubj-sum(x) #subj with normal CN
	x.hwe = hwe(x,data.type='count')
	pval = c(pval,x.hwe$p.lrt)
	maf = c(maf,min(x.hwe$allele.freq))
      }
    }

    FDR[[i]] = pval
    if(fdr)  FDR[[i]] = p.adjust(pval,method="BH")

    MAF[[i]] = maf

    hwe.reg[[1]] = udel
    hwe.reg[[2]] = FREQ[[i]][,2]
    hwe.reg[[3]] = MAF[[i]]
    hwe.reg[[4]] = FDR[[i]]
    names(hwe.reg) = c('reg','freq','maf','fdr')
    hwe.regtot[[i]] = hwe.reg
  }
  names(hwe.regtot) = paste('group',1:ng,sep="") 
  return(hwe.regtot)
} # end of function
