`read.cnv` <-
function(...,filenames=character(0),cnvfile.path=NULL,conf=TRUE,
                     annotated=FALSE,what.annotation="annotation",
                     input = c("penncnv","quantisnp","other"),sep="\t")
  {

    if(length(match.call(expand.dots=FALSE)$...) && inherits(list(...)[[1]],"cnvrun"))
      {
        object <- list(...)[[1]]
        input <- "penncnv"
        conf <- attr(object,"confidence")
        annotated <- attr(object,"annotated")
        what.annotation <- attr(object,"what.annotation")
        filenames <- object
        sep <- ifelse(attr(object,"tabout"),"\t","")
      }
    else
      {
        input <- match.arg(input)
        
        ##         if(input != "penncnv")
        ##           stop("Only penncnv methods implemented so far")
        
        
        dirfiles <- character(0)
        if(!is.null(cnvfile.path))
          dirfiles <- dir(cnvfile.path,full=TRUE)
        
        auxnames <- unlist(as.list(substitute(list(...)))[-1])
        filenames <- as.list(c(auxnames,filenames,dirfiles))
        
      }
    
    ## penncnv specific
    if(input == "penncnv")
      {
        conf.colname = "conf"
        col.names = c("chr","numsnp","length","state","sample","startsnp",
          "endsnp")
        SKIP=0
      }
    ##
    

    ## quantiSNP specific
    if(input == "quantisnp")
      {
        SKIP = 1
        conf.colname = "conf"

        ## dunno how this are treated in quantiSNP so I fix them according to the sample
        ## output file
        conf = TRUE
        annotated = FALSE
        expand.both = FALSE
        what.annotation <- NULL
        sep=""
        col.names = c("sample","chr","start","end","length","startsnp",
          "endsnp","numsnp","cn")
      }
    ##

  ## other format
    if(input == "other")
      {
        conf.colname = "conf"
        col.names = c("sample","chr","cn","startsnp","endsnp")
        SKIP=1
      }
    ##
    do.read <- function(x,conf =  conf,annotated = annotated, 
                        cnames = col.names, conf.name = conf.colname,input=input, what.annotation,
                        sep = sep, skip = SKIP)
      {
        num.cols <- scan(x,nlines=1,what="character",quiet=TRUE,skip=SKIP)
        
        
        if(!length(num.cols))
          return(NA)
        
        ncols <- length(num.cols)
        
        if(conf)
          {
            cnames <- c(cnames,conf.name)
            
            if(annotated)
              {
                cnames <- c(cnames,paste(rep(what.annotation,each=length(what.annotation)),
                                         rep(c("gene","distance"),length(what.annotation))))
                if(ncols != length(cnames))
                  stop("When arguments annotated=",annotated," (with ",length(what.annotation),
                       " annotations) and conf=",conf,
                       ", input files must have ", length(cnames) ," columns. Instead ",ncols,
                       " read in. Please check your data.")
                else
                  obj <- read.table(x,sep=sep,strip.white=TRUE,header=FALSE,as.is=TRUE,fill=TRUE,
                                    skip=SKIP,col.names=cnames)
                
              }
            else
              {
                if(ncols != (8 + 2*as.numeric(input == "quantisnp")-2*as.numeric(input == "other")))
                  stop("When arguments annotated=",annotated," and conf=",conf,
                       " input files must have", 8 + 2*as.numeric(input == "quantisnp")-2*as.numeric(input == "other"),
                       " columns. Instead ",ncols,
                       " read in. Please check your data.")
                
                obj <- read.table(x,sep=sep,strip.white=TRUE,header=FALSE,as.is=TRUE,fill=TRUE,skip=SKIP,
                                  col.names=cnames)
              }
          }
        else
          {
            if(annotated)
              {
                cnames <- c(cnames,paste(rep(what.annotation,each=length(what.annotation)),
                                        rep(c("gene","distance"),length(what.annotation))))

                if(ncols != length(cnames))
                  stop("When arguments annotated=",annotated," (with ",length(what.annotation),
                       " annotations) and conf=",conf,
                       ", input files must have ", length(cnames) ," columns. Instead ",ncols,
                       " read in. Please check your data.")
                else
                  obj <- read.table(x,sep=sep,strip.white=TRUE,header=FALSE,as.is=TRUE,fill=TRUE,
                                    skip=SKIP,col.names=cnames)
               ##                   }
              }
            else
              {
                if(ncols != length(cnames))
                  stop("When arguments annotated=",annotated," (with ",length(what.annotation),
                       "annoations) and conf=",conf,
                       ", input files must have", length(cnames) ," columns. Instead ",ncols,
                       " read in. Please check your data.")
                obj <- read.table(x,sep=sep,strip.white=TRUE,header=FALSE,as.is=TRUE,fill=TRUE,skip=SKIP,
                                  col.names=cnames)
              }
          }
        

        obj$sample <- basename(obj$sample) ## remove any possible directory information
        
        if(input == "penncnv")
          {
            obj$numsnp <- as.numeric(do.call("rbind",strsplit(obj$numsnp,split="="))[,2])
            obj$startsnp <- do.call("rbind",strsplit(obj$startsnp,split="="))[,2]
            obj$endsnp <- do.call("rbind",strsplit(obj$endsnp,split="="))[,2]
            obj$length <- do.call("rbind",strsplit(obj$length,split="="))[,2]
            obj$length <- as.numeric(gsub(",","",obj$length))
            if(conf)
              obj$conf <- as.numeric(do.call("rbind",strsplit(obj$conf,split="="))[,2])
        
        
            tt <- do.call("rbind",strsplit(obj$state,split=","))
            obj$state <- tt[,1]
            
            obj$cn <- tt[,2]
            obj$cn <- as.numeric(gsub("cn=","",obj$cn))
            
            tmp <- do.call("rbind",strsplit(obj[,"chr"],split=":"))
            obj$chr <- tmp[,1]
            tt <- do.call("rbind",strsplit(tmp[,2],split="-"))
            obj$start <- as.numeric(tt[,1])
            obj$end <- as.numeric(tt[,2])
            
          }
        else
          {
            obj$state <- NA
            obj$chr <- paste("chr",obj$chr,sep="")
          }
        
        obj <- obj[,order(names(obj))] ## put colnames in the same order
        
        name <- eval(x)
        name <- gsub("\\.txt","",name)
        name <- gsub("\\.cnv","",name)
        name <- gsub("-",".",name)
                
        obj$sample <- gsub("-",".",obj$sample)

        
        class(obj) <- c("cnvdata","data.frame")
        
        return(obj)
      } ## END do.read


##     out <- lapply(filenames,do.read,conf=conf,annotated=annotated, expand.both=expand.both,
##                   which.expand=which.expand,sep=sep,input=input)
    
    out <- lapply(filenames,do.read,conf=conf,annotated=annotated,what.annotation=what.annotation,
                  sep=sep,input=input)
    
    out <- do.call("rbind",out)
    
    attr(out,"confidence") <- conf
    attr(out,"annotated") <- annotated
    attr(out,"what.annotation") <- what.annotation
    attr(out,"input") <- input

    return(out)
    
  }

