summary.cnv <-
  function(object, annotation,test=FALSE,group=NULL,plot=FALSE,write.results=FALSE,...)
{
  ##..summarizing CNV deletion/duplications regions in 'object', optionally by group 
  ##..object = an object of class 'cnv'; output of setreg.cnv
  ##..group = the position of group variable in 'pheno.data' data.frame in 
  ##..........default=NULL -> summarize all CNV dels/dups regions simultaneously.
  ##..test = test CNV regions distribution across 'group'. default=FALSE
  ##..plot = whether to display between-group CNV regions correlation
  ## ........................  

  ## require(limma)

  if(missing(annotation))
    annotation <- eval(parse(text=attr(object,"annotation")))
  
  chr = annotation$Chr.num 
  pos = annotation$Pos
  idx = 1: length(pos)
  names(idx) = annotation$Name
  name = annotation$Name
  
  ## START of individual DELETION region summary
  ## .......................................   summary within each group

  if(is.null(group)) {
    grp = c(paste("group=","ALL",sep=""))
    ng = 1
  }  
  
  if(!is.null(group)) {
    ng = length(table(object$pheno[,group]))
    grp.var = names(table(object$pheno[,group]))
    #class(grp.var) = class(object$pheno[,group])
    grp = c(paste("group=",grp.var,sep=""))
    
  }


  ##.. for separate del/dup analysis
  if(!attr(object,"both")) {
  
   for (i in 1:length(grp)){
    cat('\n','DELETION group-by-group summary:','\n') 
    cat('\n',grp[i], '\n')
    
    pick = rep(TRUE,length(object[["del"]][["delID"]]))
    if(!is.null(group)) {
      pick = object$pheno[,group]==grp.var[i]
    }
    
    dels = unlist(object[["del"]][["delID"]][pick])   # dels = positions in the LOC.del list
    
                                        # singletons
    freq = table(dels)
                                        
    
                                        # statistics
    loc.st = object[["del"]][["loc"]][[1]][dels,"st"]
    loc.en = object[["del"]][["loc"]][[1]][dels,"en"]
    len = pos[loc.en] - pos[loc.st]
    ulen = unique(len)
    cat('Number of unique CNV regions across individuals =', length(ulen), '\n')
    cat('Average length of CNV regions', mean(ulen),'\n')
    cat('Median length of CNV regions', median(ulen),'\n')
    
    len.cut = c(0,1000,10000,50000,1000000)
    cat('Distribution of length','\n')
    print(table(cut(ulen,len.cut)))
  
   }
  
   ## summary of confidence score over all groups
   CONF = unlist(object[["del"]][["conf"]])
   REG = unlist(object[["del"]][["delID"]])  # regions
   reg.cnf = tapply(CONF, REG, median)
   reg.cnf = data.frame(del= names(reg.cnf), conf=reg.cnf)
  
   ## summary of gene annotation across all groups
   if(length(object$del$anno)>0) {
     anno = unlist(object[["del"]][["anno"]])
     anno = tapply(anno, REG, unique)
     anno = sapply(anno,paste,collapse=',')
     anno = strsplit(anno,split=',')
     anno = sapply(anno,unique)
     ucsc.anno = sapply(anno,paste,collapse=',')
   }
   ## summary of gene annotation across all groups

   ##   anno = unlist(object[["del"]][[4]])
   ##   anno = tapply(anno, REG, unique)
   ##   anno = sapply(anno,paste,collapse=',')
   ##   anno = strsplit(anno,split=',')
   ##   anno = sapply(anno,unique)
   ##   refseq.anno = sapply(anno,paste,collapse=',')
  
  
                                        # .......................................   table over groups
   FREQ = list()
   for (i in 1:ng){
    pick = rep(TRUE,length(object[["del"]][["delID"]]))
    if(!is.null(group)) {
      pick = object$pheno[,group]==grp.var[i]
    }
    
    del = unlist(object[["del"]][["delID"]][pick])   # positions in the LOC.del list
    freq = table(del)
    FREQ[[i]] = data.frame(freq)
   }
  
                                        # ............................  merge info
   if(ng==1) {
    tab = FREQ[[1]]
   }
  
   if(ng>1) {
    tab = merge(FREQ[[1]], FREQ[[2]],by='del', all=TRUE, sort=TRUE)
   }
  
   if(ng>2) {
    for(i in 3:ng) {
        tab = merge(tab, FREQ[[i]], by='del', all=TRUE, sort=TRUE)
 	colnames(tab) = c('del',grp[1:i])
    }
   }
   tab = merge(tab, reg.cnf, by='del', all=TRUE, sort=TRUE)
   tab = matrix(as.numeric(as.matrix(tab)), ncol=ng+2)
   colnames(tab) = c('del',grp,'Conf')
   ord = order(tab[,'del'])
   tab = tab[ord,]   # remove the del column
   tab = ifelse(is.na(tab), 0, tab)   # convert NA to 0
  
   sumfreq = tab[,2]
   if(ng>1) {
    sumfreq = rowSums(tab[,2:(ng+1)])
 
   }
    # CNV annotation: refer to index in LOC.del for (start,end), then ann$Name
   del = tab[,1]     # del code in tab
   st = object[["del"]][["loc"]][[1]][del,"st"]
   en = object[["del"]][["loc"]][[1]][del,"en"]
   st.snp = name[st]
   en.snp = name[en]
   len.snp = en-st+1  # number of SNPs
  
   st.pos = pos[st]
   en.pos = pos[en]  
   len = pos[en] - pos[st]
   tab.chr = chr[st]
  
    # gene.annotation in the same order as 'del' code in 'tab'
    if(length(object$del$anno)>0)
     genes.ucsc = ucsc.anno[as.character(del)]
  
  
                                        # ...............   hypothesis testing if ng>1
   if(ng>1) {
    nn = NULL
    for (i in 1:ng){
      sam.sub = unique(object$pheno[object$pheno[,group]==grp.var[i],1])
      n = length(sam.sub)
      nn = c(nn,n)
    }
   }
  
   n1 = t(tab[, 2:(ng+1)])
   PVAL = FDR = rep(NA,ncol(n1))
  
  
   if(ng>1 & test) {
    n2 = nn-n1
    for (i in 1:ncol(n1)){
      x = cbind(n1[,i], n2[,i])
      if(ng < 5) 
        pval = fisher.test(x)$p.value
      if(ng >= 5)
        pval = chisq.test(x,simulate.p.value=TRUE,B=10000)$p.value
      PVAL[i] = pval
    }

    #require(limma)
    FDR = p.adjust(PVAL,method="BH")
   }
  

   ## collating
   TAB = data.frame(Chr = tab.chr, StartSNP= st.snp, EndSNP=en.snp, NumberSNPs = len.snp,
    Start=st.pos, End = en.pos, 
    Length = len, MedConf=tab[,ng+2], 
    tab[,2:(ng+1)], Total= sumfreq, Pvalue=PVAL, FDR = FDR) 
   if(length(object$del$anno)>0) 
     TAB = data.frame(TAB,Genes.ANNO = genes.ucsc)

   if(write.results)
     write.csv(TAB, file='List of CNV DELETION regions.csv')

   DEL.OUT <- TAB
   
   ##...between-group comparison
   if(ng>1 & plot) {
    pairs(tab[,2:(ng+1)])
    title('Deletion')
    readline('Press Enter to continue..')
   }

  ## correlation between groups in term of abs/presence pattern
  if(ng>1) {    
    cat('Correlation of Deletion Regions Frequency','\n')
    print(cor(tab[,2:(ng+1)]))
  }

  
  ##................... end of DELETION summary
  
  
  ##....................START of DUPLICATION summary
  
  ## .......................................   summary within each group
  
  for (i in 1: length(grp)){
    cat('\n','DUPLICATION group-by-group summary:','\n') 
    cat('\n',grp[i], '\n')
    
    pick = rep(TRUE,length(object[["dup"]][["dupID"]]))
    if(!is.null(group)) {
      pick = object$pheno[,group]==grp.var[i]
    }
    dups = unlist(object[["dup"]][["dupID"]][pick])   # dups = positions in the LOC.dup list
    
    ## singletons
    freq = table(dups)
    ##print(table(freq))
    
    ## statistics
    loc.st = object[["dup"]][["loc"]][[1]][dups,"st"]
    loc.en = object[["dup"]][["loc"]][[1]][dups,"en"]
    len = pos[loc.en] - pos[loc.st]
    ulen = unique(len)
    cat('Number of unique CNV regions across individuals =', length(ulen), '\n')
    cat('Average length of CNV regions', mean(ulen),'\n')
    cat('Median length of CNV regions', median(ulen),'\n')
    
    len.cut = c(0,1000,10000,50000,1000000)
    cat('Distribution of length','\n')
    print(table(cut(ulen,len.cut)))
    
##     pos.a = as.numeric(names(freq))
##     loc.a = object[["dup"]][["loc"]][["locID"]][pos.a,1]
                                        # see
                                        #plot(pos[loc.a], freq, pch=2)
    
  }
  
                      # summary of confidence score over all groups
  CONF = unlist(object[["dup"]][["conf"]])
  REG = unlist(object[["dup"]][["dupID"]])  # regions
  reg.cnf = tapply(CONF, REG, median)
  reg.cnf = data.frame(dup= names(reg.cnf), conf=reg.cnf)
  
  ## summary of gene annotation across all groups
  if(length(object$dup$anno)>0) {
   anno = unlist(object[["dup"]][["anno"]])
   anno = tapply(anno, REG, unique)
   anno = sapply(anno,paste,collapse=',')
   anno = strsplit(anno,split=',')
   anno = sapply(anno,unique)
   ucsc.anno = sapply(anno,paste,collapse=',')
  }                                      # summary of gene annotation across all groups
##   anno = unlist(object[["dup"]][[4]])
##   anno = tapply(anno, REG, unique)
##   anno = sapply(anno,paste,collapse=',')
##   anno = strsplit(anno,split=',')
##   anno = sapply(anno,unique)
##   refseq.anno = sapply(anno,paste,collapse=',')
  
                                        # .......................................   table over groups -- DUP
  FREQ = list()
  for (i in 1:length(grp)){
    pick = rep(TRUE,length(object[["dup"]][["dupID"]]))
    if(!is.null(group)) {
      pick = object$pheno[,group]==grp.var[i]
    }
    
    dup = unlist(object[["dup"]][["dupID"]][pick])   # positions in the LOC.dup list
    freq = table(dup)
    FREQ[[i]] = data.frame(freq)
  }
  
                                        # ............................  merge info
  if(ng==1) {
    tab = FREQ[[1]]
  }
  
  if(ng>1) {
    tab = merge(FREQ[[1]], FREQ[[2]],by='dup', all=TRUE, sort=TRUE)
  }
  
  if(ng>2) {
    for(i in 3:ng) {
      tab = merge(tab, FREQ[[i]], by='dup', all=TRUE, sort=TRUE)
      names(tab) = c('dup',grp[1:i])
    }
  }
  tab = merge(tab, reg.cnf, by='dup', all=TRUE, sort=TRUE)
  tab = matrix(as.numeric(as.matrix(tab)), ncol=ng+2)
  colnames(tab) = c('dup',grp,'Conf')
  ord = order(tab[,'dup'])
  tab = tab[ord,]   # remove the dup column
  tab = ifelse(is.na(tab), 0, tab)   # convert NA to 0
  sumfreq = tab[,2]
  if(ng>1) {
    sumfreq = rowSums(tab[,2:(ng+1)])
  }
           # annotation: refer to index in LOC.dup for (start,end), then ann$Name
  dup = tab[,1]
  st = object[["dup"]][["loc"]][[1]][dup,"st"]
  en = object[["dup"]][["loc"]][[1]][dup,"en"]
  st.snp = name[st]
  en.snp = name[en]
  len.snp = en-st+1  # number of SNPs
  
  st.pos = pos[st]
  en.pos = pos[en]  
  len = pos[en] - pos[st]
  tab.chr = chr[st]
  
        # gene.annotation in the same order as 'dup' code in 'tab'
  if(length(object$dup$anno)>0) 
   genes.ucsc = ucsc.anno[as.character(dup)]
#  genes.refseq = refseq.anno[as.character(dup)]
  
  
  if(ng>1) {
    nn = NULL
    for (i in 1:ng){
      sam.sub = unique(object$pheno[object$pheno[,group]==grp.var[i],1])
      n = length(sam.sub)
      nn = c(nn,n)
    }
  }
  
  n1 = t(tab[, 2:(ng+1)])
  #require(limma)
  PVAL = FDR = rep(NA,ncol(n1))
  
  if(ng>1 & test) {
    n2 = nn-n1
    for (i in 1:ncol(n1)){
      x = cbind(n1[,i], n2[,i])
      if(ng<5)
        pval = fisher.test(x)$p.value
      if(ng>=5)
      if(ng >= 5)
        pval = chisq.test(x,simulate.p.value=TRUE,B=10000)$p.value
      PVAL[i] = pval
    }
    ## 
    FDR = p.adjust(PVAL,method="BH") # use 'fdr' command in package 'limma' to get adjusted pvalues.
  }
  
                                        # collating
  TAB = data.frame(Chr = tab.chr, StartSNP= st.snp, EndSNP=en.snp, NumberSNPs = len.snp,
    Start=st.pos, End = en.pos, 
    Length = len, MedConf=tab[,ng+2], 
    tab[,2:(ng+1)], Total= sumfreq, Pvalue=PVAL, FDR = FDR)

  if(length(object$dup$anno)>0) 
    TAB = data.frame(TAB, Genes.ANNO=genes.ucsc)

   if(write.results)
     write.csv(TAB, file='List of CNV DUPLICATION regions.csv')
   DUP.OUT <- TAB
  
                                        #...between-group comparison
  if(ng>1 & plot) {
    pairs(tab[,2:(ng+1)])
    title('Duplication')
    readline('Press Enter to continue..')
  }
   if(ng>1) {
    cat('Correlation of Duplication Regions Frequency','\n')
    print(cor(tab[,2:(ng+1)]))

    results <- list(Deletions=DEL.OUT,Duplications=DUP.OUT)

    
  }  #........end of DUPLICATION summary

 } # end of separate analysis

  if(attr(object,"both"))
    {
      for (i in 1:length(grp)){
        cat('\n','CNV Region group-by-group summary:','\n') 
        cat('\n',grp[i], '\n')
        
        pick = rep(TRUE,length(object[["reg"]][["regID"]]))
        if(!is.null(group)) {
          pick = object$pheno[,group]==grp.var[i]
        }
        
        regs = unlist(object[["reg"]][["regID"]][pick])   # regs = positions in the LOC.reg list
        
                                        # singletons
        freq = table(regs)
        
        
                                        # statistics
        loc.st = object[["reg"]][["loc"]][[1]][regs,"st"]
        loc.en = object[["reg"]][["loc"]][[1]][regs,"en"]
        len = pos[loc.en] - pos[loc.st]
        ulen = unique(len)
        cat('Number of unique CNV regions across individuals =', length(ulen), '\n')
        cat('Average length of CNV regions', mean(ulen),'\n')
        cat('Median length of CNV regions', median(ulen),'\n')
        
        len.cut = c(0,1000,10000,50000,1000000)
        cat('Distribution of length','\n')
        print(table(cut(ulen,len.cut)))
        
      }
      
      ## summary of confidence score over all groups
      CONF = unlist(object[["reg"]][["conf"]])
      REG = unlist(object[["reg"]][["regID"]])  # regions
      reg.cnf = tapply(CONF, REG, median)
      reg.cnf = data.frame(reg= names(reg.cnf), conf=reg.cnf)
      
      ## summary of gene annotation across all groups
      if(length(object$reg$anno)>0) {
        anno = unlist(object[["reg"]][["anno"]])
        anno = tapply(anno, REG, unique)
        anno = sapply(anno,paste,collapse=',')
        anno = strsplit(anno,split=',')
        anno = sapply(anno,unique)
        ucsc.anno = sapply(anno,paste,collapse=',')
      }
      ## summary of gene annotation across all groups
      
      ##   anno = unlist(object[["reg"]][[4]])
      ##   anno = tapply(anno, REG, unique)
      ##   anno = sapply(anno,paste,collapse=',')
      ##   anno = strsplit(anno,split=',')
      ##   anno = sapply(anno,unique)
      ##   refseq.anno = sapply(anno,paste,collapse=',')
      
      
                                        # .......................................   table over groups
      FREQ = list()
      for (i in 1:ng){
        pick = rep(TRUE,length(object[["reg"]][["regID"]]))
        if(!is.null(group)) {
          pick = object$pheno[,group]==grp.var[i]
        }
        
        reg = unlist(object[["reg"]][["regID"]][pick])   # positions in the LOC.reg list
        freq = table(reg)
        FREQ[[i]] = data.frame(freq)
      }
      
                                        # ............................  merge info
      if(ng==1) {
        tab = FREQ[[1]]
      }
      
      if(ng>1) {
        tab = merge(FREQ[[1]], FREQ[[2]],by='reg', all=TRUE, sort=TRUE)
      }
      
      if(ng>2) {
        for(i in 3:ng) {
          tab = merge(tab, FREQ[[i]], by='reg', all=TRUE, sort=TRUE)
          colnames(tab) = c('reg',grp[1:i])
        }
      }
      tab = merge(tab, reg.cnf, by='reg', all=TRUE, sort=TRUE)
      tab = matrix(as.numeric(as.matrix(tab)), ncol=ng+2)
      colnames(tab) = c('reg',grp,'Conf')
      ord = order(tab[,'reg'])
      tab = tab[ord,]   # remove the reg column
      tab = ifelse(is.na(tab), 0, tab)   # convert NA to 0
      
      sumfreq = tab[,2]
      if(ng>1) {
        sumfreq = rowSums(tab[,2:(ng+1)])
        
      }
                                        # CNV annotation: refer to index in LOC.reg for (start,end), then ann$Name
      reg = tab[,1]     # reg code in tab
      st = object[["reg"]][["loc"]][[1]][reg,"st"]
      en = object[["reg"]][["loc"]][[1]][reg,"en"]
      st.snp = name[st]
      en.snp = name[en]
      len.snp = en-st+1  # number of SNPs
      
      st.pos = pos[st]
      en.pos = pos[en]  
      len = pos[en] - pos[st]
      tab.chr = chr[st]
      
                                        # gene.annotation in the same order as 'reg' code in 'tab'
      if(length(object$reg$anno)>0)
        genes.ucsc = ucsc.anno[as.character(reg)]
      
      
                                        # ...............   hypothesis testing if ng>1
      if(ng>1) {
        nn = NULL
        for (i in 1:ng){
          sam.sub = unique(object$pheno[object$pheno[,group]==grp.var[i],1])
          n = length(sam.sub)
          nn = c(nn,n)
        }
      }
      
      n1 = t(tab[, 2:(ng+1)])
      PVAL = FDR = rep(NA,ncol(n1))
      
      
      if(ng>1 & test) {
        n2 = nn-n1
        for (i in 1:ncol(n1)){
          x = cbind(n1[,i], n2[,i])
          if(ng < 5) 
            pval = fisher.test(x)$p.value
          if(ng >= 5)
            pval = chisq.test(x,simulate.p.value=TRUE,B=10000)$p.value
          PVAL[i] = pval
        }
        
                                        #require(limma)
        FDR = p.adjust(PVAL,method="BH")
      }
      
      
      ## collating
      TAB = data.frame(Chr = tab.chr, StartSNP= st.snp, EndSNP=en.snp, NumberSNPs = len.snp,
        Start=st.pos, End = en.pos, 
        Length = len, MedConf=tab[,ng+2], 
        tab[,2:(ng+1)], Total= sumfreq, Pvalue=PVAL, FDR = FDR) 
      if(length(object$reg$anno)>0) 
        TAB = data.frame(TAB,Genes.ANNO = genes.ucsc)

      if(write.results)
        write.csv(TAB, file='List of CNV regions.csv')
      ALL.OUT <- TAB
      
      ##...between-group comparison
      if(ng>1 & plot) {
        pairs(tab[,2:(ng+1)])
        title('CNV')
        readline('Press Enter to continue..')
      }
      
      ## correlation between groups in term of abs/presence pattern
      if(ng>1) {    
        cat('Correlation of CNV Regions Frequency','\n')
        print(cor(tab[,2:(ng+1)]))
      }

      results <- list(Regions=ALL.OUT)
    } # end of simultaneous ("both") analysis
  
  return(invisible(results))
} # end of function
