\name{setreg}
\alias{setreg}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Setting common CNV regions}
\description{
Setting common CNV regions with user-specified criteria
}
\usage{
setreg(object, annotation, pheno.data, verbose = FALSE, exclude.list=NULL, 
    high.conf = 50, LIM = 0, cnv.abnormality='separate',   
    method=c('COVER','COMPOSITE'), cluster.method=c('none','single','complete','average'),
    cluster.limit=0.8)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{an object of 'cnvdata' class; usually output of \code{\link{read.cnv}.}}
  \item{annotation}{a dataframe with Probes annotation information, with the following components:
    'Name' = Probe name, 'Chr.num' = Chromosome number and 'Pos' =
    Probes position. The probes need to be sorted by chromosome and within each chromosome by 'Pos'.}
  \item{pheno.data}{a data frame of covariates with rows correspond to
    subject. The first column (\code{sample}) corresponds to subject IDs, 
    the second column (\code{gender}) corresponds to gender information (TRUE=male,FALSE=female) and the subsequent 
			columns held other covariates information.} 
  \item{verbose}{whether to print summary of CNV regions.}
  \item{exclude.list}{vector of 'bad' subjects' IDs (see pheno.data) to be excluded from analysis}
  \item{high.conf}{confidence score cut-off (in percentile) for defining highly-reliable individual regions (0-100 range). When method='COMPOSITE',
	this is the composite score cut-off (in percentile). Default=50.}
  \item{LIM}{\code{u=LIM+1} is the minimum number of individuals in a CNV region. Needed when \code{method}='COVER'.}
  \item{cnv.abnormality}{whether to consider deletion and duplication region separately when setting common CNV region or
				consider them 'both' simultaneously. Default='separate'.}
  \item{method}{if 'COVER', the COVER method will be used. If 'COMPOSITE', the COMPOSTIE method will be used (see Salim et al., 2009 for details)}
  \item{cluster.method}{if 'none'(default), clustering will not be implemented. If 'single','complete' or 'average', 
  the single linkage, complete linkage and average linkage hierarchical cluster analysis method will be implemented respectively.
  Clustering is only available when \code{cnv.abnormality}='both'(see Salim et al., 2009 for details)}
  \item{cluster.limit}{the amount of within cluster similarity required. A value of 0.8 means that members of the same cluster need to be at least 80\% similar. 
  This value must be between 0 and 1.}


}
%\details{
%  ~~ If necessary, more details than the description above ~~
%}

\value{
  If cnv.abnormality = 'both',a list of class 'cnv' with two components:
    
  \item{reg}{a list containing information on individual-specific CNV-regions. There are five components of this list:
  'regID' contains the CNV-region IDs, 'conf' contains the confidence score, 'call' contains the integer CNV calls,
  'anno' contains names of genes located within the regions and 'loc' contains information on the start and end of common regions and chromosome where
  the common regions are located. The start and end positions corresponds to indices in \code{anno.list$Name}.}

  \item{pheno}{A subset of 'pheno.data' data frame corresponds to subjects included in the analysis.} 

  \item{}{If cnv.abnormality = 'both' and cluster.method != 'none',a list of class 'cnv' with two components:}

  \item{reg}{There are eight components of this list:
  The first five components are similar to that described above, with slight modification to the naming of the CNV-region IDs. 
  The regions are named with the integer part corresponding to the ID of the original region ('mother region') had the clustering method not been used. 
  The numbers following the decimal point indicates the 'cluster ID', which is the cluster group the region falls under. 
  The next three components contain information needed for \code{plot.cnv}. 
  'plotst' and 'ploten' contain information on the genomic positions of the start and end of the individual copy number regions that overlap with the 'mother region'.
  'clustergroups' contains information on the 'cluster ID' of the individual copy number regions that overlap with the 'mother region'. }

  \item{pheno}{A subset of 'pheno.data' data frame corresponds to subjects included in the analysis.} 
  
  \item{}{If cnv.abnormality = 'separate',a list of class 'cnv' with three components: }

  \item{del}{a list containing information on individual-specific deletion-regions. There are four components of this list:
  'delID' contains the deletion-region IDs, 'conf' contains the confidence score,'anno' contains names of genes located 
  within the regions and 'loc' contains information on the start and end of common regions and chromosome where the common 
  regions are located. The start and end positions corresponds to indices in \code{anno.list$Name}.}

  \item{dup}{Similar to 'del', but instead of 'delID', it contains 'dupID' component with information on individual-specific
duplication-regions.}

  \item{pheno}{A subset of 'pheno.data' data frame corresponds to subjects included in the analysis.} 
}

\references{Salim A, Teo SM, Calza S, Ku CS, Chia KS, Pawitan, Y. Identification of common copy-number variation (CNV) regions using high-density SNP array. submitted.}
\author{Stefano Calza, Agus Salim, Yudi Pawitan}
%\note{ ~~further notes~~ 
\seealso{\code{\link{read.cnv}},\code{\link{summary.cnv}},\code{\link{hwe.cnv}}}

\examples{

myfile = file.path(system.file("data",package="cnvpack"),"hgdp_bdy.txt")

## read pennCNV output
xdat <- read.cnv(filenames=myfile,annotated=FALSE,conf=TRUE,sep="\t")

## covariates 
data(pheno)
# bad samples are the first five subjects
excl.list <- unique(pheno$sample)[1:5]
## load annotation file object 'ann'
data(ann)
out <- setreg(xdat,annotation=ann,pheno.data=pheno,exclude.list=excl.list,LIM=3,cnv.abnormality='both',method='COVER') 
hwe.out <- hwe.cnv(out)
}

% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{datagen}
