\name{summary}
\alias{summary.cnv}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Summarizing common CNV regions}
\description{
Summarizing and annotating common CNV regions and testing for association}
\usage{
\method{summary}{cnv}(object,annotation, test = FALSE, group = NULL, plot
= FALSE, write.results = FALSE...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{an object of class 'cnv'; output of \code{\link{setreg}}}
  \item{annotation}{a dataframe with Probes annotation information, with the following components:
			'Name' = Probe name, 'Chr.num' = Chromosome number and 'Pos' = Probes position.} 
  \item{test}{to test CNV regions distribution across 'group' variable. default=FALSE}
  \item{group}{the position of 'group' variable in 'pheno.data' data.frame}
  \item{plot}{to plot frequency of CNV regions between subgroups of
    subjects as defined by 'group' variable.}
  \item{write.results} If TRUE the program will write results on text files
  \item{\dots} {additional arguments affecting the summary produced}
}

\details{

  If write.results is TRUE results will written in text files.
  If cnv.abnormality = 'both', setreg
  saves the list of CNV-regions in \code{LIST of CNV Regions.csv} file
  If cnv.abnormality = 'seperate', setreg
  saves the list of deletion regions in \code{LIST of CNV Deletion Regions.csv} file and the
  list of duplication regions in \code{LIST of CNV Duplication Regions.csv} file. For each
  deletion/duplication region, the same variables will be saved (see \code{value}).


  If test=TRUE, additional columns correspond frequency of the region in each subpopulation and 
  the outcome of testing for differential frequency of the region across subpopulations, expressed
  in terms of pvalues and false discovery rate (FDR) will be provided.  \cr

  If plot=TRUE and group!=NULL,scatter plots of CNV region frequency in any two subpopulations will be produced.
}

\value{

  A list with either one (named "Regions") or two components (names
  "Deletions" and "Duplications") depending on the value of
  \code{cnv.abnormalities}: if \code{cnv.abnormalities}=="separate" the
  results will have two components.

  Each component is a dataframe with the following columns.
  
  \item{Chr}{chromosome number in which the CNV occurs with Chr=23 corresponds to the X chromosome}
  \item{StartSNP, EndSNP}{names of SNPs that mark the start and the end of the region}
  \item{NumberSNPs}{Number of SNPs in the region}
  \item{Start, End}{Position of region boundary in the genome (in bp)}
  \item{Length}{Length of the region (in bp)}
  \item{MedConf}{Median of Confidence Score for the region}
  \item{Total}{Population frequency of the region}
  \item{Genes.ANNO}{genes within the region}
  
}

\references{Salim A, Teo SM, Calza S, Ku CS, Chia KS, Pawitan, Y. Identification of common copy-number variation (CNV) regions using high-density SNP array. submitted.}
\author{Stefano Calza, Agus Salim, Yudi Pawitan}
%\note{ ~~further notes~~ 

% ~Make other sections like Warning with \section{Warning }{....} ~
%}
\seealso{\code{\link{setreg}}}
\examples{
#..setting CNV region
example(setreg)
#...summarize by gender (2nd variable in 'pheno.data' data frame).
sum <- summary(out,annotation=ann,group=2,plot=TRUE) 
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{methods}
\keyword{datagen}% __ONLY ONE__ keyword per line
